body
{ 
  background: #fff;
  color: #000;
  margin: 0px
}

/*
a:link
{ 
  background: transparent;
  color: #00f
}

a:visited
{
  background: transparent;
  color: #f00
}

a:hover
{
  background: transparent;
  color: #00f
}
*/

/* The banner that runs across the top of all nodes */
#banner
{
  margin-top/**/: 5em;
  padding-top/**/: 0.5em;
  padding-right: 10px;
  font-size: smaller;
  text-align: right;
}

/* The area underneath the banner on all nodes */
#content
{
  padding: 0em 1em 1em 1em;
}

/* The navigation bar that appears on all nodes */
#navbar
{
  position/**/: absolute;
  top/**/: 0px;
  left/**/: 0px;
  width/**/: 99%;
  padding: 0.1em 0.2em 0.1em 0.2em;
  background-color: #cfc;
  color: #000;
  font-size: 0.8em;
}

/* A group of items in the navbar */
.navbar_group
{
  float: left;
  margin: 0em 1em 0.5em 0em;
}

/* A title for that group */
.navbar_group_title
{
  display: inline;
  font-weight: bold;
}

/* An item in the navbar */
.navbar_item
{
  border-right/**/: 1px solid black;
  padding: 0em 0.5em 0em 0.5em;
}

/* The "Search" label, text field and "Go" button in the navbar */
.search_group
{
  float: right;
}

/* The "Search" label itself */
.search_label
{
  display: inline;
  font-weight: bold
}

/* The link on all nodes saying "edit this page" */
.edit_page_link
{
  float: right;
  text-align: right;
  font-weight: bold;
  font-size: 1.5em;
  padding-right: 5px;
}

/* The Recent Changes listing on the home page */
#recent
{
  float: right;
  padding: 0em 1em 1em 1em;
  border: solid #000;
  width: 250px;
  margin: 10px;
  font-size: 0.9em
}

/* An item title in the listing */
#recent dt
{
  font-weight: bold
}

/* An item definition in the listing */
#recent dd
{
}

/* The name of a node */
.node_name
{
}

/* Listing of categories and locales for a node */
#categories_and_locales
{
  font-size: small
}

/* The widget for distance lookups */
#find_within_distance
{
}

/* The listing of coordinates on nodes with geo metadata */
#coordinates
{
}

/* The node version indicator */
#version
{
  border-top: 1px solid #999;
  margin-top: 10px;
  font-size: small;
  padding-top: 5px;
}

/* The 'RDF/XML for this node' link that appears on all nodes */
#rdf_link
{
  font-size: small;
  padding-top: 5px;
  float: right;
}

/* The search form that appears on all nodes */
#search_form
{
  border-top: 1px solid #999;
  padding-top: 5px;
}

/* The footer that appears on all nodes */
#footer
{
  margin-top: 10px;
  padding-top: 10px;
  font-size: 0.8em;
  text-align: right
}

/* Submit buttons in forms */
input.form_button
{
  font-size: 0.9em;
  font-style: bold;
  background-color: #fff;
  color: #000
}

/* The 'before' part of a diff listing */
.diff1
{
  background-color: yellow;
}

/* The 'after' part of a diff listing */
.diff2
{
  background-color: pink;
}
