body
{ 
  background: #fff;
  color: #000;
  margin: 0px
}

a:link
{ 
  background: transparent;
  color: #00f
}

a:visited
{
  background: transparent;
  color: #f00
}

a:hover
{
  background: transparent;
  color: #00f
}

/* The banner that runs across the top of all nodes */
#banner
{
  padding: 10px;
  background-color: #ccc;
  color: #fff;
  border-bottom: 1px solid #000
}

/* The area underneath the banner on all nodes */
#content
{
  padding: 1em
}

/* The navigation bar that appears on all nodes */
#navbar
{
  float: right;
  width: 175px;
  padding: 10px;
  border: 1px solid #000;
  background-color: #fff;
  color: #000;
  font-size: 0.8em;
  margin: 0px 0px 10px 10px
}

/* A group of items in the navbar */
.navbar_group
{
}

/* A title for that group */
.navbar_group_title
{
  margin: 1em 0em 0em 0em;
  font-weight: bold
}

/* An item in the navbar */
.navbar_item
{
  margin-top: 0em;
  margin-bottom: 0em
}

/* The "Search" label, text field and "Go" button in the navbar */
.search_group
{
}
 
/* The "Search" label itself */
.search_label
{
  font-weight: bold
}

/* The link on all nodes saying "edit this page" */
.edit_page_link
{
  font-weight: bold;
  margin-bottom: 1em;
  padding-top: 1em;
  border-top: 1px solid #000
}

/* The Recent Changes listing on the home page */
#recent
{
  float: right;
  padding: 1em;
  border: 1px dashed #666;
  width: 250px;
  margin: 10px;
  font-size: 0.8em
}

/* An item title in the listing */
#recent dt
{
  font-weight: bold
}

/* An item definition in the listing */
#recent dd
{
}

/* The name of a node */
.node_name
{
  font-style: italic
}

/* Listing of categories and locales for a node */
#categories_and_locales
{
  font-size: small
}

/* The widget for distance lookups */
#find_within_distance
{
}

/* The listing of coordinates on nodes with geo metadata */
#coordinates
{
}

/* The node version indicator */
#version
{
  margin-top: 10px;
  font-size: small
}

/* The 'RDF/XML for this node' link that appears on all nodes */
#rdf_link
{
  font-size: small;
  float: right
}

/* The search form that appears on all nodes */
#search_form
{
  padding-top: 10px;
  border-top: 1px solid #999
}

/* The footer that appears on all nodes */
#footer
{
  margin-top: 10px;
  padding-top: 10px;
  border-top: 1px solid #999;
  font-size: 0.8em;
  text-align: right
}

/* Submit buttons in forms */
input.form_button
{
  font-size: 0.9em;
  font-style: bold;
  background-color: #fff;
  color: #000
}

/* The 'before' part of a diff listing */
.diff1
{
  background-color: yellow;
}

/* The 'after' part of a diff listing */
.diff2
{
  background-color: pink;
}

/* Hidden elements */
.hidden
{
  display: none;
}

/* Labels for form elements */
.label
{
  font-weight: bold;
}