use Test::More tests => 8;

use_ok( "OpenGuides" );
use_ok( "OpenGuides::CGI" );
use_ok( "OpenGuides::Config" );
use_ok( "OpenGuides::Feed" );
use_ok( "OpenGuides::RDF" );
use_ok( "OpenGuides::Search" );
use_ok( "OpenGuides::Template" );
use_ok( "OpenGuides::Utils" );
